//+------------------------------------------------------------------+
//|                                        wpr + bollinger bands.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1  DeepSkyBlue
#property indicator_color2  DimGray
#property indicator_color3  DimGray
#property indicator_color4  DimGray
#property indicator_style2  STYLE_DOT
#property indicator_style3  STYLE_DOT
#property indicator_style4  STYLE_DOT
#property indicator_width1  2
#property indicator_maximum 0
#property indicator_minimum -100

//
//
//
//
//

extern string TimeFrame       = "Current time frame";
extern int    WprPeriod       = 14;
extern int    BandsPeriod     = 14;
extern double BandsDeviations = 1.0;  
extern bool   Interpolate     = true;

//
//
//
//
//

double wpr[];
double bandsUp[];
double bandsMi[];
double bandsDo[];

string IndicatorFileName;
bool   CalculatingWpr;
bool   ReturningBars;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,wpr);
   SetIndexBuffer(1,bandsUp);
   SetIndexBuffer(2,bandsMi);
   SetIndexBuffer(3,bandsDo);

   //
   //
   //
   //
   //
   
      IndicatorFileName   = WindowExpertName();
      CalculatingWpr = (TimeFrame=="CalculateWpr"); if (CalculatingWpr) return(0);
      ReturningBars  = (TimeFrame=="returnBars");   if (ReturningBars)  return(0);
      timeFrame      = stringToTimeFrame(TimeFrame);
         
   //
   //
   //
   //
   //
               
   IndicatorShortName("Wpr with bands "+timeFrameToString(timeFrame)+" ("+WprPeriod+","+BandsPeriod+","+DoubleToStr(BandsDeviations,2)+")");
   return(0);
}
int deinit() { return(0); }



//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;
   
   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
           limit = MathMin(Bars-counted_bars,Bars-1);
           if (ReturningBars)  { wpr[0] = limit; return(0); }

   //
   //
   //
   //
   //
   
      if (timeFrame > Period()) limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));
      if (CalculatingWpr)
      {
         for(i=limit; i>=0; i--) wpr[i] = iWPR(NULL,0,WprPeriod,i);
         for(i=limit; i>=0; i--) 
         {
            double deviations = iStdDevOnArray(wpr,0,BandsPeriod,0,MODE_SMA,i);
                   bandsMi[i] = iMAOnArray(wpr,0,BandsPeriod,0,MODE_SMA,i);
                   bandsUp[i] = bandsMi[i]+deviations*BandsDeviations;
                   bandsDo[i] = bandsMi[i]-deviations*BandsDeviations;
         }
         return(0);
      }

   //
   //
   //
   //
   //

      for(i=limit; i>=0; i--)
      {
         int y = iBarShift(NULL,timeFrame,Time[i]);
            wpr[i]     = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateWpr",WprPeriod,BandsPeriod,BandsDeviations,0,y);
            bandsUp[i] = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateWpr",WprPeriod,BandsPeriod,BandsDeviations,1,y);
            bandsMi[i] = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateWpr",WprPeriod,BandsPeriod,BandsDeviations,2,y);
            bandsDo[i] = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateWpr",WprPeriod,BandsPeriod,BandsDeviations,3,y);

         //
         //
         //
         //
         //
      
         if (timeFrame <= Period() || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
         if (!Interpolate) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            double factor = 1.0 / n;
            for(int k = 1; k < n; k++)
            {
  	            wpr[i+k]     = k*factor*wpr[i+n]     + (1.0-k*factor)*wpr[i];
  	            bandsUp[i+k] = k*factor*bandsUp[i+n] + (1.0-k*factor)*bandsUp[i];
  	            bandsMi[i+k] = k*factor*bandsMi[i+n] + (1.0-k*factor)*bandsMi[i];
  	            bandsDo[i+k] = k*factor*bandsDo[i+n] + (1.0-k*factor)*bandsDo[i];
            }  	            
      }
      
   //
   //
   //
   //
   //
      
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int cchar = StringGetChar(s, length);
         if((cchar > 96 && cchar < 123) || (cchar > 223 && cchar < 256))
                     s = StringSetChar(s, length, cchar - 32);
         else if(cchar > -33 && cchar < 0)
                     s = StringSetChar(s, length, cchar + 224);
   }
   return(s);
}